"""Integration tests for InternetBS"""
import os
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class InternetbsProviderTests(TestCase, IntegrationTests):
    """TestCase for InternetBS"""
    provider_name = 'internetbs'
    domain = 'world-of-ysera.com'

    def _filter_query_parameters(self):
        return ['ApiKey', 'Password']

    def _test_parameters_overrides(self):
        # workaround ENV problems during testing
        env_key = os.environ.get('LEXICON_INTERNETBS_AUTH_KEY')
        env_password = os.environ.get('LEXICON_INTERNETBS_AUTH_PASSWORD')
        return {'auth_key': env_key, 'auth_password': env_password}



"""Module provider for RcodeZero"""
from __future__ import absolute_import
import hashlib
import json
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['rcode0.net']


def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-token", help="specify token for authentication")


class Provider(BaseProvider):
    """Provider class for Cloudflare"""

    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self._zone_data = None
        self.api_endpoint = 'https://my.rcodezero.at/api/v1'

    def _authenticate(self):
        if self._zone_data is None:
            self._zone_data = self._get('/zones/' + self.domain)

        self.domain_id = self.domain

    # Create record.

    def _create_record(self, rtype, name, content):
        rname = self._fqdn_name(name)
        newcontent = self._clean_content(rtype, content)

        updated_data = {
            'name': rname,
            'type': rtype,
            'records': [],
            'ttl': self._get_lexicon_option('ttl') or 600,
            'changetype': 'ADD'
        }

        updated_data['records'].append(
            {'content': newcontent, 'disabled': False})

        payload = self._get(
            '/zones/{0}/rrsets?page_size=-1'.format(self.domain_id))

        for rrset in payload['data']:
            if rrset['name'] == rname and rrset['type'] == rtype:
                updated_data['ttl'] = rrset['ttl']

                for record in rrset['records']:
                    if record['content'] != newcontent:
                        updated_data['records'].append(
                            {
                                'content': record['content'],
                                'disabled': record['disabled']
                            })
                    updated_data['changetype'] = 'UPDATE'
                break

        request = [updated_data]
        LOGGER.debug('request: %s', request)

        self._patch('/zones/' + self.domain + '/rrsets', data=request)
        return True

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        filter_obj = {'per_page': 100}
        if rtype:
            filter_obj['type'] = rtype
        if name:
            filter_obj['name'] = self._full_name(name)
        if content:
            filter_obj['content'] = content

        payload = self._get(
            '/zones/{0}/rrsets?page_size=-1'.format(self.domain_id))

        records = []
        for rrset in payload['data']:
            if (name is None or self._fqdn_name(rrset['name']) == self._fqdn_name(
                    name)) and (rtype is None or rrset['type'] == rtype):
                for record in rrset['records']:
                    if content is None or record['content'] == self._clean_content(rtype, content):
                        # rcode0 does not have a record id, so lets create one
                        processed_record = {
                            'type': rrset['type'],
                            'name': self._full_name(rrset['name']),
                            'ttl': rrset['ttl'],
                            'content': self._unclean_content(rrset['type'], record['content']),
                            'id': self._make_identifier(rrset['type'],
                                                        rrset['name'], record['content'])
                        }
                        records.append(processed_record)

        LOGGER.debug('list_records: %s', records)
        return records

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        self._delete_record(identifier, rtype, name, None)
        return self._create_record(rtype, name, content)

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):

        LOGGER.debug("delete %s %s %s %s", identifier, rtype, name, content)
        if identifier is None and (rtype is None or name is None):
            raise Exception("Must specify at least id or  both rtype and name")

        payload = self._get(
            '/zones/{0}/rrsets?page_size=-1'.format(self.domain_id))

        if identifier is not None:
            rtype, name, content = self._parse_identifier(identifier, payload)

        update_data = None
        for rrset in payload['data']:
            if rrset['type'] == rtype and self._fqdn_name(rrset['name']) == self._fqdn_name(name):
                update_data = rrset

                if content is None:
                    update_data['records'] = []
                    update_data['changetype'] = 'DELETE'
                else:
                    new_record_list = []
                    for record in update_data['records']:
                        if self._clean_content(rrset['type'], content) != record['content']:
                            new_record_list.append(record)

                    update_data['records'] = new_record_list
                    if new_record_list:
                        update_data['changetype'] = 'UPDATE'
                    else:
                        update_data['changetype'] = 'DELETE'
                break

        if update_data is not None:
            request = [update_data]
            LOGGER.debug('request: %s', request)
            self._patch('/zones/' + self.domain + '/rrsets', data=request)

        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        response = requests.request(action, self.api_endpoint + url, params=query_params,
                                    data=json.dumps(data),
                                    headers={
                                        'Authorization': 'Bearer ' +
                                                         self._get_provider_option(
                                                             'auth_token'),
                                        'Content-Type': 'application/json'
                                    })
        # if the request fails for any reason, throw an error.
        if response.status_code >= 400:
            LOGGER.error('Bad Request: %s', response.text)
        response.raise_for_status()
        return response.json()

    # generate a unique id for a give record
    def _make_identifier(self, rtype, name, content):  # pylint: disable=no-self-use
        sha256 = hashlib.sha256()
        sha256.update(('type=' + rtype + ',').encode('utf-8'))
        sha256.update(('name=' + name + ',').encode('utf-8'))
        sha256.update(('content=' + content + ',').encode('utf-8'))
        return sha256.hexdigest()[0:7]

    def _parse_identifier(self, identifier, payload):  # pylint: disable=no-self-use

        for rrset in payload['data']:
            for record in rrset['records']:
                if self._make_identifier(
                        rrset['type'], rrset['name'], record['content']) == identifier:
                    rtype = rrset['type']
                    name = self._full_name(rrset['name'])
                    content = self._unclean_content(
                        rrset['type'], record['content'])
                    return rtype, name, content

        raise Exception("Record with ID {} not found ".format(identifier))


    def _clean_content(self, rtype, content):
        if rtype in ("TXT", "LOC"):
            if content[0] != '"':
                content = '"' + content
            if content[-1] != '"':
                content += '"'
        elif rtype == "CNAME":
            content = self._fqdn_name(content)
        return content

    def _unclean_content(self, rtype, content):
        if rtype in ("TXT", "LOC"):
            content = content.strip('"')
        elif rtype == "CNAME":
            content = self._full_name(content)
        return content



"""Implement a provider for Zilore (https://zilore.com)"""
import logging

import requests

from lexicon.providers.base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['zilore.net']


def provider_parser(subparser):
    """Construct a subparser for Zilore provider"""
    subparser.description = '''
    Zilore API requires an API key that can be found in your Zilore profile, at the API tab.
    The API access is available only for paid plans.
    '''
    subparser.add_argument('--auth-key', help='specify the Zilore API key to use')


class Provider(BaseProvider):
    """
    Construct the Zilore provider

    Zilore API is very clean and well constructed. All features required to make direct
    CRUD operations are present, including update, and record ids. The filters to list
    records given in the specification seems to not work, but can be easily done on Lexicon
    side using list comprehension on the provided arguments on _list_records.

    Authentication is done by passing the API key specific to a user in the X-Auth-Key header.
    """
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None

    def _authenticate(self):
        result = self._get('/domains')

        target_domain = [item for item in result['response']
                         if item['domain_name'] == self.domain]

        if not target_domain:
            raise Exception('Domain {0} is not available on this account'.format(self.domain))

        self.domain_id = target_domain[0]['domain_id']

    def _list_records(self, rtype=None, name=None, content=None):
        result = self._get('/domains/{0}/records'.format(self.domain), {})

        records = [self._clean_TXT_record({
            'id': item['record_id'],
            'type': item['record_type'],
            'name': self._full_name(item['record_name']),
            'content': item['record_value'],
            'ttl': item['record_ttl']
        }) for item in result['response']]

        if rtype:
            records = [record for record in records if record['type'] == rtype]
        if name:
            records = [record for record in records if record['name'] == self._full_name(name)]
        if content:
            records = [record for record in records if record['content'] == content]

        LOGGER.debug('list_records: %s', records)

        return records

    def _create_record(self, rtype, name, content):
        if not rtype or not name or not content:
            raise Exception(
                'Error, rtype, name and content are mandatory to create a record.')

        records = self._list_records(rtype, name, content)

        if records:
            LOGGER.debug('not creating a duplicate record: %s', records[0])
            return True

        record = {
            'record_type': rtype,
            'record_name': self._full_name(name),
            'record_value': content if rtype != 'TXT' else '"{0}"'.format(content)
        }

        if self._get_lexicon_option('ttl'):
            record['record_ttl'] = self._get_lexicon_option('ttl')

        result = self._post('/domains/{0}/records'.format(self.domain), query_params=record)

        LOGGER.debug('create_record: %s', result['response']['record_id'])

        return True

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        if not identifier and (not rtype and not name):
            raise Exception(
                'Error, identifier or rtype+name parameters are required.')

        if identifier:
            records = self._list_records()
            records_to_update = [record for record in records if record['id'] == identifier]
        else:
            records_to_update = self._list_records(rtype=rtype, name=name)

        if not records_to_update:
            raise Exception('Error, could not find a record matching the request.')

        if len(records_to_update) > 1:
            LOGGER.warning(
                'Warning, multiple records found for given parameters, '
                'only first one will be updated: %s', records_to_update)

        record = records_to_update[0]
        update = {
            'record_type': rtype if rtype else record['type'],
            'record_name': self._full_name(name) if name else record['name'],
            'record_ttl': self._get_lexicon_option('ttl')
                          if self._get_lexicon_option('ttl') else record['ttl']
        }

        if content:
            update['record_value'] = content if rtype != 'TXT' else '"{0}"'.format(content)

        result = self._put('/domains/{0}/records/{1}'.format(self.domain, record['id']),
                           query_params=update)

        LOGGER.debug('update_record: %s', result['response']['record_id'])

        return True

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        if identifier:
            records = self._list_records()
            records_to_delete = [record for record in records if record['id'] == identifier]
        else:
            records_to_delete = self._list_records(rtype=rtype, name=name, content=content)

        if not records_to_delete:
            raise Exception('Error, could not find a record matching the request.')

        for record in records_to_delete:
            self._delete('/domains/{0}/records'.format(self.domain),
                         query_params={'record_id': record['id']})

            LOGGER.debug('delete_record: %s %s %s %s', identifier, rtype, name, content)

        return True

    def _request(self, action='GET', url='/', data=None, query_params=None):
        response = requests.request(action, 'https://api.zilore.com/dns/v1{0}'.format(url),
                                    params=query_params, json=data,
                                    headers={'X-Auth-Key': self._get_provider_option('auth_key')})

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            LOGGER.error('Content of error response:')
            LOGGER.error(response.json())
            raise error

        return response.json()



"""Module provider for Netcup"""
from __future__ import absolute_import
import json
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['netcup.de']


def provider_parser(subparser):
    """Configure provider parser for Netcup"""
    subparser.add_argument(
        "--auth-customer-id", help="specify customer number for authentication")
    subparser.add_argument(
        "--auth-api-key", help="specify API key for authentication")
    subparser.add_argument(
        "--auth-api-password", help="specify API password for authentication")


class Provider(BaseProvider):
    """Provider class for Netcup"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = 0
        self.zone_ttl = None
        self.api_session_id = None
        self.api_endpoint = (
            self._get_provider_option('api_endpoint') or
            'https://ccp.netcup.net/run/webservice/servers/endpoint.php?JSON')

    # lexicon.provider.Provider overrides:

    def _authenticate(self):
        """Authenticate with netcup server. Must be called first."""
        login_info = self._apicall('login')
        self.api_session_id = login_info['apisessionid']
        if not self.api_session_id:
            raise Exception('Login failed')
        # query ttl and verify access to self.domain:
        zone_info = self._apicall('infoDnsZone', domainname=self.domain)
        self.zone_ttl = zone_info['ttl']

    def _create_record(self, rtype, name, content):
        """Create record. If it already exists, do nothing."""
        if not self._list_records(rtype, name, content):
            self._update_records([{}], {
                'type': rtype,
                'hostname': self._relative_name(name),
                'destination': content,
                'priority': self._get_lexicon_option('priority'),
            })
        LOGGER.debug('create_record: %s', True)
        return True

    def _list_records(self, rtype=None, name=None, content=None):
        """List all records. Return an empty list if no records found.
        ``rtype``, ``name`` and ``content`` are used to filter records."""
        records = [
            {
                'id': record['id'],
                'type': record['type'],
                'name': self._full_name(record['hostname']),
                'content': record['destination'],
                'priority': record['priority'],
                'ttl': self.zone_ttl,
            }
            for record in self._raw_records(None, rtype, name, content)
        ]
        LOGGER.debug('list_records: %s', records)
        return records

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        """Create or update a record."""
        records = self._raw_records(identifier, rtype, name, content)
        self._update_records(records, {
            'type': rtype,
            'hostname': self._relative_name(name),
            'destination': content,
            'priority': self._get_lexicon_option('priority'),
        })
        LOGGER.debug('update_record: %s', True)
        return True

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        """Delete an existing record. If record does not exist, do nothing."""
        records = self._raw_records(identifier, rtype, name, content)
        LOGGER.debug('delete_records: %s', [rec['id'] for rec in records])
        self._update_records(records, {
            'deleterecord': True,
            'type': rtype,
            'hostname': name,
            'destination': content,
        })
        LOGGER.debug('delete_record: %s', True)
        return True

    # Helpers

    def _raw_records(self, identifier=None, rtype=None, name=None, content=None):
        """Return list of record dicts in the netcup API convention."""
        record_fields = {
            'id': identifier,
            'type': rtype,
            'hostname': name and self._relative_name(name),
            'destination': content,
        }
        # type/hostname/destination of the dnsrecord type are mandatory (even
        # when deleting), and must be queried if not all were specified:
        if all(record_fields.values()):
            return [record_fields]
        data = self._apicall('infoDnsRecords', domainname=self.domain)
        records = data.get('dnsrecords', [])
        return [
            record for record in records
            if all(record[k] == v for k, v in record_fields.items() if v)
        ]

    def _update_records(self, records, data):
        """Insert or update a list of DNS records, specified in the netcup API
        convention.

        The fields ``hostname``, ``type``, and ``destination`` are mandatory
        and must be provided either in the record dict or through ``data``!
        """
        data = {k: v for k, v in data.items() if v}
        records = [dict(record, **data) for record in records]
        return self._apicall(
            'updateDnsRecords',
            domainname=self.domain,
            dnsrecordset={'dnsrecords': records},
        ).get('dnsrecords', [])

    def _apicall(self, method, **params):
        """Call an API method and return response data. For more info, see:
        https://ccp.netcup.net/run/webservice/servers/endpoint"""
        LOGGER.debug('%s(%r)', method, params)
        auth = {
            'customernumber': self._get_provider_option('auth_customer_id'),
            'apikey': self._get_provider_option('auth_api_key'),
        }
        if method == 'login':
            auth['apipassword'] = self._get_provider_option('auth_api_password')
        else:
            auth['apisessionid'] = self.api_session_id
        if not all(auth.values()):
            raise Exception('No valid authentication mechanism found')
        data = self._request('POST', url='', data={
            'action': method,
            'param': dict(params, **auth),
        })
        if data['status'] != 'success':
            raise Exception("{} ({})".format(
                data['longmessage'], data['statuscode']))
        return data.get('responsedata', {})

    def _request(self, action='GET', url='/', data=None, query_params=None):
        """Perform network request to configured JSON endpoint."""
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        default_headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }
        response = requests.request(
            action,
            self.api_endpoint + url,
            data=json.dumps(data),
            params=query_params,
            headers=default_headers)
        response.raise_for_status()
        return response.json()



"""Module provider for Conoha"""
from __future__ import absolute_import
import json
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['conoha.io']


def provider_parser(subparser):
    """Configure provider parser for Conoha"""
    subparser.add_argument(
        "--auth-region", help="specify region. If empty, region `tyo1` will be used.")
    subparser.add_argument(
        "--auth-token",
        help=("specify token for authentication. If empty, the username "
              "and password will be used to create a token."))
    subparser.add_argument(
        "--auth-username",
        help="specify api username for authentication. Only used if --auth-token is empty.")
    subparser.add_argument(
        "--auth-password",
        help="specify api user password for authentication. Only used if --auth-token is empty.")
    subparser.add_argument(
        "--auth-tenant-id",
        help="specify tenand id for authentication. Only used if --auth-token is empty.")


class Provider(BaseProvider):
    """Provider class for Conoha"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None

        self.api_endpoint = ('https://dns-service.{0}.conoha.io/v1'
                             .format(self._get_provider_option('region') or 'tyo1'))
        self.auth_api_endpoint = ('https://identity.{0}.conoha.io/v2.0'
                                  .format(self._get_provider_option('region') or 'tyo1'))
        self.auth_token = None

    # Authenticate against provider,
    # Make any requests required to get the domain's id for this provider,
    # so it can be used in subsequent calls.
    # Should throw an error if authentication fails for any reason,
    # of if the domain does not exist.
    def _authenticate(self):
        self.auth_token = self._get_provider_option('auth_token')
        if not self.auth_token:
            if not (self._get_provider_option('auth_username')
                    and self._get_provider_option('auth_password')):
                raise Exception(
                    "auth_username and auth_password or auth_token must be specified.")
            auth_response = self._send_request(
                'POST', '{0}/tokens'
                .format(self.auth_api_endpoint), {
                    'auth': {
                        'passwordCredentials': {
                            'username': self._get_provider_option('auth_username'),
                            'password': self._get_provider_option('auth_password')
                        },
                        'tenantId': self._get_provider_option('auth_tenant_id')
                    }
                })
            self.auth_token = auth_response['access']['token']['id']

        payload = self._get('/domains', {
            'name': self._fqdn_name(self.domain)
        })

        if not payload['domains']:
            raise Exception('No domain found')
        if len(payload['domains']) > 1:
            raise Exception('Too many domains found. This should not happen')

        self.domain_id = payload['domains'][0]['id']

    # Create record. If record already exists with the same content, do nothing'
    def _create_record(self, rtype, name, content):
        if not rtype:
            raise Exception("rtype must be specified.")
        if not name:
            raise Exception("name must be specified.")
        if not content:
            raise Exception("content must be specified.")
        if not self._get_lexicon_option('priority') and rtype in ("MX", "SRV"):
            raise Exception("priority must be specified.")

        try:
            self._post('/domains/{0}/records'.format(self.domain_id),
                       self._record_payload(rtype, name, content))
        except requests.exceptions.HTTPError as err:
            # 409 Duplicate Record
            if err.response.status_code != 409:
                raise err

        LOGGER.debug('create_record: %s', True)
        return True

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        payload = self._get('/domains/{0}/records'.format(self.domain_id))
        records = payload['records']

        if rtype:
            records = [record for record in records if record['type'] == rtype]
        if name:
            records = [record for record in records if record['name']
                       == self._fqdn_name(name)]
        if content:
            records = [
                record for record in records if record['data'] == content]

        records = [{
            'type': record['type'],
            'name': self._full_name(record['name']),
            'ttl': record['ttl'],
            'content': record['data'],
            'id': record['id']
        } for record in records]

        LOGGER.debug('list_records: %s', records)
        return records

    # Update a record. Identifier must be specified.
    def _update_record(self, identifier, rtype=None, name=None, content=None):
        if not identifier:
            records = self._list_records(rtype, name)
            if len(records) != 1:
                raise Exception("Cannot determine record")
            identifier = records[0]['id']

        self._put('/domains/{0}/records/{1}'
                  .format(self.domain_id, identifier), self._record_payload(rtype, name, content))

        LOGGER.debug('update_record: %s', True)
        return True

    # Delete an existing record.
    # If record does not exist, do nothing.
    # If an identifier is specified, use it, otherwise do a lookup using type, name and content.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        records = self._list_records(rtype, name, content)

        if identifier:
            records = [
                record for record in records if record['id'] == identifier]

        for record in records:
            self._delete(
                '/domains/{0}/records/{1}'.format(self.domain_id, record['id']))

        LOGGER.debug('delete_record: %s', True)
        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        return self._send_request(action, '{0}{1}'.format(self.api_endpoint, url),
                                  data, query_params)

    def _send_request(self, action, url, data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        response = requests.request(
            action, url, data=json.dumps(data), params=query_params, headers={
                'X-Auth-Token': self.auth_token,
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            })
        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        if response.headers['content-type'].startswith('application/json'):
            return response.json()
        return response.text

    def _record_name(self, name):  # pylint: disable=no-self-use
        return '%s.' % name.rstrip('.') if name else None

    def _record_payload(self, rtype, name, content):
        priority = self._get_lexicon_option('priority')
        ttl = self._get_lexicon_option('ttl')
        return {
            'name': self._fqdn_name(name) if name else None,
            'type': rtype,
            'data': self._record_name(content) if rtype in (
                "CNAME", "MX", "NS", "SRV") else content,
            'priority': int(priority) if priority else None,
            'ttl': int(ttl) if ttl else None
        }



"""Integration tests for Transip"""
import os
from tempfile import mkstemp
from unittest import TestCase

import pytest
from lexicon.tests.providers.integration_tests import IntegrationTests


FAKE_KEY = b"""
-----BEGIN RSA PRIVATE KEY-----
MIIEpAIBAAKCAQEAxV08IlJRwNq9WyyGO2xRyT0F6XIBD2R5CrwJoP7gIHVU/Mhk
KeK8//+MbUZtKFoeJi9lI8Cbkqe7GVk9yab6R2/vVzV21XRh+57R79nEh+QTf/vZ
dg+DjUn62U4lcgoVp3sHddIi/Zi58xz2a2lGGIdolsv1x0/PmAQPULt721IG/osp
RBjTtaZ8niXrOTfjH814i8kgXu74CCGu0X6kJBIezMA2wqY1ZKZYRMpfrxkEZe0t
45pEM1CmSTCqyDMpwYou9wJaDHn0ts1KvKkKBfmO4B0nqfW9Sv9rkmpBCLTtMobj
dQ8EwWv1L1g9uddkPALgRODEpR4fq7PTmq2VEQIDAQABAoIBAFf4wwEZaE9qMNUe
94YtNhdZF/WCV26g/kMGpdQZR5WwNv2l5N+2rT/+jH140tcVtDKZFZ/mDnJESWV3
Hc9wmkaVYj2hGyLyCWq61CDxFGTuCLMXc0roh17HBwUtjAtU62oHsL+XtvkKxnfT
BRPDjPcKBFiS+S6qKII97QWzS/XpxL47VpXcYboVunzUncIKghC93LdvPp3ukh6x
HIarqyctqkksLJtLgH5ffuABCJLChetpOIfcfspjtMoji43CXXd7Y3rGWy3EzSHA
s4mNb4K6r8MOlJj3HiTn9bEgL2V2q3OHSYHYXexir67vkQeN+NsC80G0uODt6Uuo
Cd1RobECgYEA+O+nZYRc22jI8oqRoQeCx6cTWJoaf4OYDXcaerRMIiE7yigHNgmX
LGs9RYTVrWXzjM5KHVvPvavpm/zIBoa5fA7uqdH9BjuZVLm1COXzKxF5hevZuAxr
zGQWDbdvzdsihPBvwlf0dKScA/WIRW0KCqUmC6IlS/An4Y0nI05P+KsCgYEAyvby
cfUPgeanBnYE3GGou3cLiurzvK3vHuQl6vVE3DcheUj/5tKTwG5Q3/7y51MKHnfH
xEc/X2IePXYVy0JwpC6NHzkyJPuJ1zYlkQGSs81TUbYOk9SKi3SL9bM+3vRzYFoL
GMLJuvEqIscxLNqR0xQB5eBkg8T+AVJiA7cTITMCgYEAn5/ND2OYx3ihoiUIzOEs
EyonVaE7bJjNX5UH/bavOxNka3TPau8raOg7GeDbw5ykV53QGJNO2qjp24R0Hvs0
5UAN+gcU4HJHF/UdCN+q1esWqbFaopIUbbOgEJuXrcDembAzecM8la8X+9Ht19bb
oYfUpZELqW4NpKwGdLU6wpECgYAfn3hI3xjKcYiGji7Vs3WZt8OZol/VfvgpxPxP
bmWLNh/GCOSuLxMMQWPicpOgDSUfeCQs5bjvAJebleFxaOmp+wLL4Zp5fqOMX4hc
3nTgBNa9fXMp/0ySy9besk3SaR3s3jqqYfcSZG7fOk/kIC3mSFC/Y0Xl7fRxekeB
Mq4NVwKBgQDQ+3+cgZph5geq0PUuKMvMECDuCEnG8rrr4jTCe+sRP34y1IaxJ2w6
S6p+kvTBePSqV2wWZCls6p7mhGEto+8H9b4pWdmSqccn0vFu4kekm/OU4+IxqzWQ
KPeh76yhdzsFwzh+0LBPfkFgFn3YlHp0eoywNpm57MFxWx8u3U2Hkw==
-----END RSA PRIVATE KEY-----
"""


# The following fields were removed from the test fixtures:
#   getInfo: contacts, authcode, registrationDate, renewalDate
# using:
#   find tests/fixtures/cassettes/transip/
#       -name \*.json -exec sed -i 's/<contacts.*<\/contacts>//g' '{}' \;
#   find tests/fixtures/cassettes/transip/
#       -name \*.json -exec sed -i 's/<authCode.*<\/authCode>//g' '{}' \;
#   find tests/fixtures/cassettes/transip/
#       -name \*.json -exec sed -i 's/<registrationDate.*<\/registrationDate>//g' '{}' \;
#   find tests/fixtures/cassettes/transip/
#       -name \*.json -exec sed -i 's/<renewalDate.*<\/renewalDate>//g' '{}' \;


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from define_tests.TheTests
class TransipProviderTests(TestCase, IntegrationTests):
    """TestCase for Transip"""
    provider_name = 'transip'
    domain = 'hurrdurr.nl'

    # Disable setUp and tearDown, and set a real username and key in
    # provider_opts to execute real calls
    def _test_parameters_overrides(self):
        (_fake_fd, _fake_key) = mkstemp()
        _fake_file = os.fdopen(_fake_fd, 'wb', 1024)
        _fake_file.write(FAKE_KEY)
        _fake_file.close()
        self._fake_key = _fake_key  # pylint: disable=attribute-defined-outside-init

        options = {
            'auth_username': 'foo',
            'auth_api_key': _fake_key
        }

        return options

    def tearDown(self):
        try:
            os.unlink(self._fake_key)
        except AttributeError:
            # Method _test_options may not have been executed,
            # in this case self._fake_key does not exist.
            pass

    def _filter_headers(self):
        return ['Cookie']

    @pytest.mark.skip(reason="manipulating records by id is not supported")
    def test_provider_when_calling_delete_record_by_identifier_should_remove_record(self):
        return

    @pytest.mark.skip(reason=("adding docs.example.com as a CNAME target will result in a RFC 1035 error"))  # pylint: disable=line-too-long
    def test_provider_when_calling_create_record_for_CNAME_with_valid_name_and_content(self):
        return



"""Integration tests for Zilore"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests, _vcr_integration_test  # pylint: disable=protected-access
from lexicon.providers.zilore import Provider


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class ZiloreProviderTests(TestCase, IntegrationTests):
    """TestCase for Zeit"""
    Provider = Provider
    provider_name = 'zilore'
    domain = 'full4ir.tk'

    def _filter_headers(self):
        return ['X-Auth-Key']

    # We override this test because Zilore refuses to create a A record with '127.0.0.1' value.
    @_vcr_integration_test
    def test_provider_when_calling_create_record_for_A_with_valid_name_and_content(self):  # pylint: disable=invalid-name
        provider = self._construct_authenticated_provider()
        assert provider.create_record('A', 'localhost', '1.1.1.1')



"""Module provider for Dinahosting"""
from __future__ import absolute_import
#import re
import hashlib
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['dinahosting.com']

# These record types are either not supported by the API or are non-compliant
UNSUPPORTED_TYPES = ['MX', 'SRV', 'NS', 'SOA', 'LOC']

# The API returns the record's content in any of these keys
CONTENT_KEYS = ['ip', 'address', 'text', 'destinationHostname']

# The API returns the record's name in any of these keys
NAME_KEYS = ['host', 'hostname']

# API response codes
RC_SUCCESS = 1000
RC_SUCCESS_PENDING = 1001
RC_UNKNOWN_COMMAND = 2000
RC_COMMAND_SYNTAX_ERROR = 2001
RC_COMMAND_USE_ERROR = 2002
RC_PARAM_MISSING = 2003
RC_PARAM_VALUE_RANGE = 2004
RC_PARAM_VALUE_SYNTAX = 2005
RC_AUTH_ERROR_USER = 2200
RC_AUTH_ERROR_OBJECT = 2201
RC_OBJECT_EXISTS = 2302
RC_OBJECT_NOT_EXISTS = 2303
RC_COMMAND_FAILED = 2400
RC_COMMAND_FAILED_FATAL = 2500
RC_COMMAND_TIMEOUT = 2501


def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-username", help="specify username for authentication")
    subparser.add_argument(
        "--auth-password", help="specify password for authentication")


class Provider(BaseProvider):
    """Provider class for Dinahosting"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://dinahosting.com/special/api.php'

    def _authenticate(self):
        data = {
            'command': 'Services_GetDomains'
        }
        payload = self._post('', data)

        if not payload['data']:
            raise Exception('No domain found')
        domain_found = False
        for data in payload['data']:
            if data['domain'] == self.domain:
                domain_found = True
                break
        if not domain_found:
            raise Exception('Requested domain is not among the owned domains')

        self.domain_id = self.domain

    def _create_record(self, rtype, name, content):
        Provider._check_unsupported_type(rtype)

        rel_name = self._relative_name(name)
        data = {
            'domain': self.domain,
            'hostname': rel_name
        }

        if rtype == 'A':
            data['command'] = "Domain_Zone_AddTypeA"
            data['ip'] = content
        elif rtype == 'AAAA':
            data['command'] = "Domain_Zone_AddTypeAAAA"
            data['ip'] = content
        elif rtype == 'CNAME':
            data['command'] = "Domain_Zone_AddTypeCname"
            data['destinationHostname'] = content
        elif rtype == 'TXT':
            data['command'] = "Domain_Zone_AddTypeTXT"
            data['text'] = content

        try:
            self._post('', data)
        except requests.exceptions.HTTPError as err:
            already_exists = ((err.response.json()['responseCode'] == RC_OBJECT_EXISTS) or
                              (err.response.json()['responseCode'] == RC_COMMAND_FAILED))
            if not already_exists:
                raise

        LOGGER.debug('create_record: %s', True)
        return True

    def _list_records(self, rtype=None, name=None, content=None):
        if rtype:
            formatted_rtype = "Cname" if rtype == 'CNAME' else rtype
            data = {
                'command': 'Domain_Zone_GetType{0}'.format(formatted_rtype),
                'domain': self.domain
            }
        else:
            data = {
                'command': 'Domain_Zone_GetAll',
                'domain': self.domain
            }

        payload = self._post('', data)

        records = []
        for record in payload['data']:
            precord = self._parse_record(record)
            ptype = rtype if rtype else precord['type']

            if name and name not in precord['name_variants']:
                continue

            if content and content != precord['content']:
                continue

            if rtype and precord['type'] and rtype != precord['type']:
                continue

            processed_record = {
                'type': ptype,
                'name': precord['name_variants'][0],
                'content': precord['content'],
                'ttl': None # The API does not expose the record's ttl
            }
            processed_record['id'] = Provider._identifier(processed_record)
            records.append(processed_record)

        LOGGER.debug('list_records: %s', records)
        return records

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        if identifier:
            records = self._list_records()
            matching_records = [record for record in records if record['id'] == identifier]
        else:
            matching_records = self._list_records(rtype, name)

        # Check if record exists, otherwise raise an error
        if not matching_records:
            raise Exception("No matching records found. Cannot update.")
        if len(matching_records) > 1:
            raise Exception("Multiple matching records found. Cannot update.")

        record = matching_records[0]
        # The API does not offer update functions for most of the record types
        delete_result = self._delete_record(identifier, record['type'],
                                            record['name'], record['content'])
        create_result = self._create_record(rtype, name, content)

        LOGGER.debug('update_record: %s', (delete_result and create_result))
        return delete_result and create_result

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        if identifier:
            records = self._list_records()
            delete_records = [record for record in records if record['id'] == identifier]
        else:
            delete_records = self._list_records(rtype, name, content)

        for delete_record in delete_records:
            ptype = rtype if rtype else delete_record['type']
            Provider._check_unsupported_type(ptype)

            rel_name = self._relative_name(delete_record['name'])
            data = {
                'domain': self.domain,
                'hostname': rel_name
            }

            if ptype == 'A':
                data['command'] = "Domain_Zone_DeleteTypeA"
                data['ip'] = delete_record['content']
            elif ptype == 'AAAA':
                data['command'] = "Domain_Zone_DeleteTypeAAAA"
                data['ip'] = delete_record['content']
            elif ptype == 'CNAME':
                data['command'] = "Domain_Zone_DeleteTypeCname"
            elif ptype == 'TXT':
                data['command'] = "Domain_Zone_DeleteTypeTXT"
                data['text'] = delete_record['content']

            try:

                self._post('', data)
            except requests.exceptions.HTTPError as err:
                not_exists = (err.response.json()['responseCode'] == RC_OBJECT_NOT_EXISTS)
                if not not_exists:
                    raise

        LOGGER.debug('delete_record: %s', True)
        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        # All requests are POST. Query parameters are not used.
        query_params = {}

        if data is None:
            data = {}
        data['responseType'] = 'Json'

        username = self._get_provider_option('auth_username')
        password = self._get_provider_option('auth_password')
        response = requests.request(action,
                                    self.api_endpoint + url,
                                    auth=(username, password),
                                    params=query_params,
                                    data=data)
        # If the request fails for any reason, throw an error.
        response.raise_for_status()

        rjson = response.json()
        if int(rjson['responseCode']) != RC_SUCCESS:
            api_error_message = u'%s API Error for url: %s' % (rjson['responseCode'], response.url)
            raise requests.exceptions.HTTPError(api_error_message, response=response)
        return rjson

    def _parse_record(self, record):
        parsed_record = {}
        parsed_record['type'] = Provider._parse_record_type(record)
        parsed_record['content'] = Provider._find_matching_record_value(CONTENT_KEYS, record)
        raw_name = Provider._find_matching_record_value(NAME_KEYS, record)
        fqdn_name = self._fqdn_name(raw_name)
        full_name = self._full_name(raw_name)
        parsed_record['name_variants'] = [full_name, fqdn_name, raw_name]

        return parsed_record

    @staticmethod
    def _identifier(record):
        md5 = hashlib.md5()
        md5.update((record.get('type', '')).encode('utf-8'))
        md5.update((record.get('name', '')).encode('utf-8'))
        md5.update((record.get('data', '')).encode('utf-8'))
        return md5.hexdigest()

    @staticmethod
    def _find_matching_record_value(keys, record):
        if record is None or keys is None:
            return None

        matched_value = None
        for key in keys:
            if key in record:
                matched_value = record[key]
                break
        return matched_value

    @staticmethod
    def _parse_record_type(record):
        if not record or 'type' not in record:
            return None
        if record['type'].startswith('MX'):
            return 'MX'
        return record['type']

    @staticmethod
    def _check_unsupported_type(rtype):
        if rtype in UNSUPPORTED_TYPES:
            raise Exception('Record type {0} is not supported by the API'.format(rtype))



"""Module provider for CloudXNS"""
from __future__ import absolute_import
try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode
import hashlib
import json
import logging
import time

import requests

from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['cloudxns.net']


def provider_parser(subparser):
    """Generate subparser for CloudXNS"""
    subparser.add_argument(
        "--auth-username", help="specify API-KEY for authentication")
    subparser.add_argument(
        "--auth-token", help="specify SECRET-KEY for authentication")


class Provider(BaseProvider):
    """Provider class for CloudXNS"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://www.cloudxns.net/api2'

    def _authenticate(self):

        payload = self._get('/domain')
        for record in payload['data']:
            if record['domain'] == self.domain + '.':
                self.domain_id = record['id']
                break

        if self.domain_id is None:
            raise Exception('No domain found')

    # Create record. If record already exists with the same content, do nothing'
    def _create_record(self, rtype, name, content):

        record = {
            'domain_id': self.domain_id,
            'host': self._relative_name(name),
            'value': content,
            'type': rtype,
            'line_id': 1,
        }
        if self._get_lexicon_option('ttl'):
            record['ttl'] = self._get_lexicon_option('ttl')

        try:
            self._post('/record', record)
        except requests.exceptions.HTTPError as err:
            already_exists = (err.response.json()['code'] == 34)
            if not already_exists:
                raise

        # CloudXNS will return bad HTTP Status when error, will throw at
        # r.raise_for_status() in _request()
        LOGGER.debug('create_record: %s', True)
        return True

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        payload = self._get('/record/' + self.domain_id,
                            {'host_id': 0, 'offset': 0, 'row_num': 2000})
        records = []
        for record in payload['data']:
            processed_record = {
                'type': record['type'],
                'name': self._full_name(record['host']),
                'ttl': record['ttl'],
                'content': record['value'],
                # this id is useless unless your doing record linking. Lets return the
                # original record identifier.
                'id': record['record_id']
            }
            if processed_record['type'] == 'TXT':
                processed_record['content'] = processed_record['content'].replace(
                    '"', '')
                # CloudXNS will add quotes automaticly for TXT records,
                # https://www.cloudxns.net/Support/detail/id/114.html
            records.append(processed_record)

        if rtype:
            records = [record for record in records if record['type'] == rtype]
        if name:
            records = [record for record in records if record['name']
                       == self._full_name(name)]
        if content:
            records = [
                record for record in records if record['content'] == content]

        LOGGER.debug('list_records: %s', records)
        return records

    # Create or update a record.
    def _update_record(self, identifier, rtype=None, name=None, content=None):

        if not identifier:
            records = self._list_records(name=name)
            if len(records) == 1:
                identifier = records[0]['id']
            else:
                raise Exception('Record identifier could not be found.')

        data = {
            'domain_id': self.domain_id,
            'host': self._relative_name(name),
            'value': content,
            'type': rtype
        }
        if self._get_lexicon_option('ttl'):
            data['ttl'] = self._get_lexicon_option('ttl')

        self._put('/record/' + identifier, data)

        LOGGER.debug('update_record: %s', True)
        return True

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_id = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_id = [record['id'] for record in records]
        else:
            delete_record_id.append(identifier)

        LOGGER.debug('delete_records: %s', delete_record_id)

        for record_id in delete_record_id:
            self._delete('/record/' + record_id + '/' + self.domain_id)

        # is always True at this point, if a non 200 response is returned an error is raised.
        LOGGER.debug('delete_record: %s', True)
        return True

    # Helpers

    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        data['login_token'] = self._get_provider_option(
            'auth_username') + ',' + self._get_provider_option('auth_token')
        data['format'] = 'json'
        if query_params:
            query_string = '?' + urlencode(query_params)
        else:
            query_string = ''
            query_params = {}
        if data:
            data = json.dumps(data)
        else:
            data = ''
        date = time.strftime('%a %b %d %H:%M:%S %Y', time.localtime())
        default_headers = {
            'API-KEY': self._get_provider_option('auth_username'),
            'API-REQUEST-DATE': date,
            'API-HMAC': hashlib.md5(
                "{0}{1}{2}{3}{4}{5}{6}".format(
                    self._get_provider_option('auth_username'),
                    self.api_endpoint,
                    url,
                    query_string,
                    data,
                    date,
                    self._get_provider_option('auth_token')).encode('utf-8')).hexdigest(),
            'API-FORMAT': 'json'}
        default_auth = None
        response = requests.request(action, self.api_endpoint + url, params=query_params,
                                    data=data, headers=default_headers, auth=default_auth)
        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        return response.json()



"""Integration tests for GratisDNS"""
from unittest import TestCase

import re
# import pytest
from lexicon.tests.providers.integration_tests import IntegrationTests
from lexicon.providers.gratisdns import Provider


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class GratisDNSProviderTests(TestCase, IntegrationTests):
    """TestCase for GratisDNS"""
    Provider = Provider
    provider_name = 'gratisdns'
    domain = 'denisa.dk'
    gratisdns_session = '0123456789abcdef0123456789abcdef'

    def _filter_post_data_parameters(self):
        return ['login', 'password']

    def _filter_headers(self):
        return ['Cookie']

    def _replace_auth(self, cookie):
        cookie = re.sub('ORGID=.*;',
                        'ORGID={};'.format(self.gratisdns_session),
                        cookie)
        return cookie

    # Inspired by thehover provider
    def _filter_response(self, response):
        if 'basestring' not in globals():
            basestring = str

        if 'set-cookie' in response['headers']:
            if isinstance(response['headers']['set-cookie'], basestring):
                response['headers']['set-cookie'] = \
                    self._replace_auth(response['headers']['set-cookie'])
            else:
                for i, cookie in enumerate(response['headers']['set-cookie']):
                    response['headers']['set-cookie'][i] = self._replace_auth(cookie)

        return response
